/* -*- Mode:C++; c-file-style:"gnu"; indent-tabs-mode:nil; -*- */
/*
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation;
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Copyright 2011 Rob Smits, Divam Jain, Sarah Pidcock, Ian Goldberg
 */

class TcpSession {
  public:
    TcpSession(){};
    TcpSession(uint32_t _ip1, uint32_t _ip2, uint16_t _port1, uint16_t _port2) {
      setData(_ip1, _ip2, _port1, _port2);
    }

    void setData(uint32_t _ip1, uint32_t _ip2, uint16_t _port1, uint16_t _port2) {
      ip1 = _ip1;
      ip2 = _ip2;
      port1 = _port1;
      port2 = _port2;
    }

    static void getOrdering(const TcpSession &sess, uint32_t &largest_ip, uint32_t &smallest_ip, uint16_t &largest_ip_port, uint16_t &smallest_ip_port) {
      if (sess.ip1 == sess.ip2) {
        if (sess.port1 > sess.port2) {
          largest_ip = sess.ip1;
          largest_ip_port = sess.port1;
          smallest_ip = sess.ip2;
          smallest_ip_port = sess.port2;
        } else {
          largest_ip = sess.ip2;
          largest_ip_port = sess.port2;
          smallest_ip = sess.ip1;
          smallest_ip_port = sess.port1;
        }
      } else if (sess.ip1 > sess.ip2) {
        largest_ip = sess.ip1;
        largest_ip_port = sess.port1;
        smallest_ip = sess.ip2;
        smallest_ip_port = sess.port2;
      } else {
        largest_ip = sess.ip2;
        largest_ip_port = sess.port2;
        smallest_ip = sess.ip1;
        smallest_ip_port = sess.port1;
      }
    }

    bool operator==(const TcpSession &other) const {
      //return !(*this < other) && !(other < *this);
      bool result = true;

      uint32_t our_largest_ip, our_smallest_ip;
      uint32_t other_largest_ip, other_smallest_ip;

      uint16_t our_largest_ip_port, our_smallest_ip_port;
      uint16_t other_largest_ip_port, other_smallest_ip_port;

      //order the IP addresses within each object
      getOrdering(*this, our_largest_ip, our_smallest_ip, our_largest_ip_port, our_smallest_ip_port);
      getOrdering(other, other_largest_ip, other_smallest_ip, other_largest_ip_port, other_smallest_ip_port);

      result &= our_largest_ip == other_largest_ip;
      result &= our_smallest_ip == other_smallest_ip;
      result &= our_largest_ip_port == other_largest_ip_port;
      result &= our_smallest_ip_port == other_smallest_ip_port;

      return result;
    }

    bool operator<(const TcpSession &other) const {
      bool result = true;

      uint32_t our_largest_ip, our_smallest_ip;
      uint32_t other_largest_ip, other_smallest_ip;

      uint16_t our_largest_ip_port, our_smallest_ip_port;
      uint16_t other_largest_ip_port, other_smallest_ip_port;

      //order the IP addresses within each object
      getOrdering(*this, our_largest_ip, our_smallest_ip, our_largest_ip_port, our_smallest_ip_port);
      getOrdering(other, other_largest_ip, other_smallest_ip, other_largest_ip_port, other_smallest_ip_port);

      //This is a mess -- we can switch types to a pair of pairs to avoid making our own operator here
      if (our_largest_ip == other_largest_ip) {
        if (our_largest_ip_port == other_largest_ip_port) {
          if (our_smallest_ip == other_smallest_ip) {
            if (our_smallest_ip_port == other_smallest_ip_port) {
              result = false;
            } else {
              result = our_smallest_ip_port < other_smallest_ip_port;
            }
          } else {
            result = our_smallest_ip <  other_smallest_ip;
          }
        } else {
            result = our_largest_ip_port <  other_largest_ip_port;
        }
      } else {
        result =  our_largest_ip < other_largest_ip;
      }
      return result;
    }


  private:
    uint32_t ip1;
    uint32_t ip2;
    uint16_t port1;
    uint16_t port2;
};

